@@ -301,14 +301,15 @@ def write_table(
     if spark is None:
         spark = get_spark_session("upsert_data")
 
-    stream = (
-        df.writeStream.trigger(once=True)
-        .format("delta")
-        .option("checkpointLocation", checkpoint_location)
-        .option("header", "true")
-        .option("mergeSchema", "true")
-        .options(**write_options)
-    )
+    if insert_mode != "upsert":
+        stream = (
+            df.writeStream.trigger(once=True)
+            .format("delta")
+            .option("checkpointLocation", checkpoint_location)
+            .option("header", "true")
+            .option("mergeSchema", "true")
+            .options(**write_options)
+        )
 
     table_exists = check_table_exists(
         spark=spark, db_name=db_name, table_name=table_name
@@ -318,6 +319,16 @@ def write_table(
         truncate_table(spark=spark, table_name=f"{db_name}.{table_name}")
 
     if sort_fields and upsert_fields and insert_mode=="upsert":
+
+        stream = (
+            df.write.trigger(once=True)
+            .format("delta")
+            .option("checkpointLocation", checkpoint_location)
+            .option("header", "true")
+            .option("mergeSchema", "true")
+            .options(**write_options)
+        )
+
         if table_exists:
             print(
                 f"TABLE EXIST - upsert_fields: {upsert_fields} - sort_fields: {sort_fields}"
@@ -508,6 +519,7 @@ def landing_to_bronze(
     output_db_name=None,
     tbl_properties=None,
     upsert_fields: list = None,
+    input_df=None
 ):
     """
     Perform the ingestion of data into the bronze_layer database
@@ -558,16 +570,19 @@ def landing_to_bronze(
     if insert_mode is None:
         insert_mode = "append"
 
-    df = read_data(
-        spark,
-        data_input_path=data_input_path,
-        dict_schema_or_struct_schema=schema,
-        data_type=data_type,
-        read_options=read_options,
-        infer_schema=infer_schema,
-        data_pack_name=data_pack_name,
-        dataset_name=dataset_name,
-    )
+    if not input_df:
+        df = read_data(
+                spark,
+                data_input_path=data_input_path,
+                dict_schema_or_struct_schema=schema,
+                data_type=data_type,
+                read_options=read_options,
+                infer_schema=infer_schema,
+                data_pack_name=data_pack_name,
+                dataset_name=dataset_name,
+            )
+    else:
+        df = input_df
 
     if type(schema) == dict:
         df = process_dataframe(df, js_schema=schema)
@@ -643,7 +658,8 @@ def bronze_to_silver(
     tbl_properties=None,
     sort_fields: list = None,
     upsert_fields: list = None,
-    governance_options=None
+    governance_options=None,
+    input_df=None
 ):
     if spark is None:
         spark = get_spark_session(app_name=dataset_name)
@@ -679,9 +695,12 @@ def bronze_to_silver(
             spark=spark, db_name=input_db_name, table_name=input_table_name
         )
 
-    df = read_table(
-        spark, db_name=input_db_name, table_name=input_table_name, where=where
-    )
+    if not input_df:
+        df = read_table(
+            spark, db_name=input_db_name, table_name=input_table_name, where=where
+        )
+    else:
+        df = input_df
 
     if df.first() == None:
         print("Dataframe is empty")
@@ -805,6 +824,7 @@ def silver_to_gold(
     where=None,
     tbl_properties=None,
     governance_options=None,
+    input_df=None
 ):
     if spark is None:
         spark = get_spark_session(app_name=dataset_name)
@@ -840,14 +860,17 @@ def silver_to_gold(
     if insert_mode is None:
         insert_mode = "append"
 
-    if insert_mode == "upsert":
-        df = read_table_no_stream(
-            spark, db_name=input_db_name, table_name=input_table_name, where=where
-        )
+    if not input_df:
+        if insert_mode == "upsert":
+            df = read_table_no_stream(
+                spark, db_name=input_db_name, table_name=input_table_name, where=where
+            )
+        else:
+            df = read_table(
+                spark, db_name=input_db_name, table_name=input_table_name, where=where
+            )
     else:
-        df = read_table(
-            spark, db_name=input_db_name, table_name=input_table_name, where=where
-        )
+        df = input_df
 
     if df.first() == None:
         print("Dataframe is empty")
